package com.spatialdev.osm.model;

import android.test.InstrumentationTestCase;

import com.spatialdev.osm.model.OSMNode;
import com.spatialdev.osm.model.OSMDataSet;
import com.spatialdev.osm.model.OSMElement;
import com.spatialdev.osm.model.OSMRelation;
import com.spatialdev.osm.model.OSMWay;
import com.spatialdev.osm.model.OSMXmlParser;
import com.spatialdev.osm.model.OSMXmlWriter;

import java.io.InputStream;
import java.util.Collection;
import java.util.LinkedList;
import java.util.Map;

/**
 * Created by Nicholas Hallahan on 1/16/15.
 * nhallahan@spatialdev.com* 
 */
public class OSMXmlWriterTest extends InstrumentationTestCase {
    
    private static final String USER = "theoutpost";
    private static final String EMPTY_XML = "<?xml version='1.0' encoding='UTF-8' ?><osm version=\"0.6\" generator=\"OpenMapKit 0.1\" user=\"theoutpost\" />";
    private static final String BALLARD_WAYS_XML = "<?xml version='1.0' encoding='UTF-8' ?><osm version=\"0.6\" generator=\"OpenMapKit 0.1\" user=\"theoutpost\"><way id=\"6427328\" version=\"4\" changeset=\"4304484\" timestamp=\"2010-04-02T16:59:20Z\"><nd ref=\"53183015\" /><nd ref=\"53179273\" /><tag k=\"highway\" v=\"residential\" /><tag k=\"name\" v=\"Northwest Fern Place\" /><tag k=\"tiger:cfcc\" v=\"A41\" /><tag k=\"tiger:county\" v=\"King, WA\" /><tag k=\"tiger:name_base\" v=\"Fern\" /><tag k=\"tiger:name_direction_prefix\" v=\"NW\" /><tag k=\"tiger:name_type\" v=\"Pl\" /><tag k=\"tiger:separated\" v=\"no\" /><tag k=\"tiger:source\" v=\"tiger_import_dch_v0.6_20070830\" /><tag k=\"tiger:tlid\" v=\"186580852\" /><tag k=\"tiger:zip_left\" v=\"98107\" /><tag k=\"tiger:zip_right\" v=\"98107\" /></way><way id=\"6434079\" version=\"8\" changeset=\"26404331\" timestamp=\"2014-10-29T03:41:18Z\"><nd ref=\"53189011\" /><nd ref=\"53230823\" /><nd ref=\"3096003520\" /><nd ref=\"53189008\" /><tag k=\"highway\" v=\"residential\" /><tag k=\"name\" v=\"7th Avenue Northwest\" /><tag k=\"tiger:cfcc\" v=\"A41\" /><tag k=\"tiger:county\" v=\"King, WA\" /><tag k=\"tiger:name_base\" v=\"7th\" /><tag k=\"tiger:name_direction_suffix\" v=\"NW\" /><tag k=\"tiger:name_type\" v=\"Ave\" /><tag k=\"tiger:zip_left\" v=\"98107\" /><tag k=\"tiger:zip_right\" v=\"98107\" /></way><way id=\"6440053\" version=\"6\" changeset=\"4296062\" timestamp=\"2010-04-01T17:33:05Z\"><nd ref=\"504911067\" /><nd ref=\"680023522\" /><nd ref=\"504911069\" /><nd ref=\"60979342\" /><nd ref=\"53179275\" /><tag k=\"highway\" v=\"residential\" /><tag k=\"name\" v=\"Northwest 43rd Street\" /><tag k=\"tiger:cfcc\" v=\"A41\" /><tag k=\"tiger:county\" v=\"King, WA\" /><tag k=\"tiger:name_base\" v=\"43rd\" /><tag k=\"tiger:name_direction_prefix\" v=\"NW\" /><tag k=\"tiger:name_type\" v=\"St\" /><tag k=\"tiger:separated\" v=\"no\" /><tag k=\"tiger:source\" v=\"tiger_import_dch_v0.6_20070830\" /><tag k=\"tiger:tlid\" v=\"186580874\" /><tag k=\"tiger:upload_uuid\" v=\"bulk_upload.pl-29a9466e-9467-44bf-9700-aac8a269b22a\" /><tag k=\"tiger:zip_left\" v=\"98107\" /><tag k=\"tiger:zip_right\" v=\"98107\" /></way><way id=\"6442659\" version=\"5\" changeset=\"4304484\" timestamp=\"2010-04-02T17:13:16Z\"><nd ref=\"53179276\" /><nd ref=\"67569725\" /><nd ref=\"53197114\" /><nd ref=\"53197113\" /><nd ref=\"53095342\" /><tag k=\"highway\" v=\"residential\" /><tag k=\"name\" v=\"Northwest Bright Street\" /><tag k=\"tiger:cfcc\" v=\"A41\" /><tag k=\"tiger:county\" v=\"King, WA\" /><tag k=\"tiger:name_base\" v=\"Bright\" /><tag k=\"tiger:name_direction_prefix\" v=\"NW\" /><tag k=\"tiger:name_type\" v=\"St\" /><tag k=\"tiger:separated\" v=\"no\" /><tag k=\"tiger:source\" v=\"tiger_import_dch_v0.6_20070830\" /><tag k=\"tiger:tlid\" v=\"186580848:186580849:186580870:186580997:186580873\" /><tag k=\"tiger:zip_left\" v=\"98107\" /><tag k=\"tiger:zip_right\" v=\"98107\" /></way><way id=\"6443222\" version=\"12\" changeset=\"25695467\" timestamp=\"2014-09-26T21:24:59Z\"><nd ref=\"53191895\" /><nd ref=\"1833204869\" /><nd ref=\"53105926\" /><nd ref=\"53148454\" /><nd ref=\"53179278\" /><nd ref=\"2889672608\" /><nd ref=\"53197114\" /><nd ref=\"53194837\" /><nd ref=\"53197491\" /><nd ref=\"53189540\" /><nd ref=\"67569743\" /><nd ref=\"53106426\" /><nd ref=\"53197490\" /><nd ref=\"53161542\" /><nd ref=\"504911046\" /><nd ref=\"53161553\" /><nd ref=\"53149116\" /><nd ref=\"67569769\" /><nd ref=\"67569766\" /><nd ref=\"67569764\" /><nd ref=\"53197489\" /><tag k=\"highway\" v=\"secondary\" /><tag k=\"lanes\" v=\"4\" /><tag k=\"maxspeed\" v=\"30 mph\" /><tag k=\"name\" v=\"Leary Way Northwest\" /><tag k=\"tiger:cfcc\" v=\"A41\" /><tag k=\"tiger:county\" v=\"King, WA\" /><tag k=\"tiger:name_base\" v=\"Leary\" /><tag k=\"tiger:name_direction_suffix\" v=\"NW\" /><tag k=\"tiger:name_type\" v=\"Way\" /><tag k=\"tiger:zip_left\" v=\"98107\" /><tag k=\"tiger:zip_right\" v=\"98107\" /></way><way id=\"6502043\" version=\"4\" changeset=\"4304484\" timestamp=\"2010-04-02T18:14:25Z\"><nd ref=\"53243384\" /><nd ref=\"53189547\" /><nd ref=\"53183015\" /><tag k=\"highway\" v=\"residential\" /><tag k=\"name\" v=\"9th Avenue Northwest\" /><tag k=\"tiger:cfcc\" v=\"A41\" /><tag k=\"tiger:county\" v=\"King, WA\" /><tag k=\"tiger:name_base\" v=\"9th\" /><tag k=\"tiger:name_direction_suffix\" v=\"NW\" /><tag k=\"tiger:name_type\" v=\"Ave\" /><tag k=\"tiger:separated\" v=\"no\" /><tag k=\"tiger:source\" v=\"tiger_import_dch_v0.6_20070830\" /><tag k=\"tiger:tlid\" v=\"186705870:186580851\" /><tag k=\"tiger:zip_left\" v=\"98107\" /><tag k=\"tiger:zip_right\" v=\"98107\" /></way><way id=\"9121527\" version=\"7\" changeset=\"20799217\" timestamp=\"2014-02-26T22:16:20Z\"><nd ref=\"53179276\" /><nd ref=\"2691856852\" /><nd ref=\"67569717\" /><nd ref=\"53179275\" /><nd ref=\"53179274\" /><nd ref=\"67569707\" /><nd ref=\"1752970231\" /><nd ref=\"53179273\" /><tag k=\"highway\" v=\"residential\" /><tag k=\"name\" v=\"8th Avenue Northwest\" /><tag k=\"tiger:cfcc\" v=\"A41\" /><tag k=\"tiger:county\" v=\"King, WA\" /><tag k=\"tiger:name_base\" v=\"8th\" /><tag k=\"tiger:name_direction_suffix\" v=\"NW\" /><tag k=\"tiger:name_type\" v=\"Ave\" /><tag k=\"tiger:zip_left\" v=\"98177\" /><tag k=\"tiger:zip_right\" v=\"98177\" /></way><way id=\"9121533\" version=\"8\" changeset=\"26404331\" timestamp=\"2014-10-29T03:41:21Z\"><nd ref=\"53189547\" /><nd ref=\"53189008\" /><nd ref=\"3155658707\" /><nd ref=\"968442280\" /><nd ref=\"53189541\" /><nd ref=\"53189540\" /><nd ref=\"53095338\" /><nd ref=\"53112769\" /><nd ref=\"53189539\" /><nd ref=\"2492631291\" /><nd ref=\"566896206\" /><nd ref=\"53175453\" /><tag k=\"highway\" v=\"residential\" /><tag k=\"name\" v=\"Northwest 42nd Street\" /><tag k=\"tiger:cfcc\" v=\"A41\" /><tag k=\"tiger:county\" v=\"King, WA\" /><tag k=\"tiger:name_base\" v=\"42nd\" /><tag k=\"tiger:name_direction_prefix\" v=\"NW\" /><tag k=\"tiger:name_type\" v=\"St\" /><tag k=\"tiger:zip_left\" v=\"98107\" /><tag k=\"tiger:zip_right\" v=\"98107\" /></way><way id=\"41291798\" version=\"4\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:40Z\"><nd ref=\"504911054\" /><nd ref=\"504911053\" /><nd ref=\"538422791\" /><nd ref=\"504911057\" /><nd ref=\"2889672632\" /><nd ref=\"504911056\" /><nd ref=\"538422799\" /><nd ref=\"538422801\" /><nd ref=\"538422777\" /><nd ref=\"538422779\" /><nd ref=\"2889672618\" /><nd ref=\"2889672626\" /><nd ref=\"538422755\" /><nd ref=\"538422757\" /><nd ref=\"504911052\" /><nd ref=\"2889672611\" /><nd ref=\"538422759\" /><nd ref=\"538422832\" /><nd ref=\"504911068\" /><nd ref=\"2889669699\" /><nd ref=\"504911067\" /><tag k=\"access\" v=\"yes\" /><tag k=\"highway\" v=\"service\" /><tag k=\"surface\" v=\"asphalt\" /></way><way id=\"41291800\" version=\"3\" changeset=\"4296062\" timestamp=\"2010-04-01T17:33:05Z\"><nd ref=\"504911066\" /><nd ref=\"504911065\" /><nd ref=\"504911064\" /><nd ref=\"504911074\" /><nd ref=\"504911073\" /><nd ref=\"504911072\" /><nd ref=\"504911071\" /><nd ref=\"538422850\" /><nd ref=\"504911069\" /><tag k=\"highway\" v=\"service\" /><tag k=\"service\" v=\"driveway\" /><tag k=\"surface\" v=\"asphalt\" /></way><way id=\"41291802\" version=\"1\" changeset=\"2565517\" timestamp=\"2009-09-22T07:08:21Z\"><nd ref=\"53183015\" /><nd ref=\"504911068\" /><tag k=\"highway\" v=\"service\" /></way><way id=\"41292345\" version=\"3\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:40Z\"><nd ref=\"504916926\" /><nd ref=\"504916925\" /><nd ref=\"504916924\" /><nd ref=\"2889669693\" /><nd ref=\"504916922\" /><nd ref=\"2889672633\" /><nd ref=\"504916920\" /><nd ref=\"504916919\" /><nd ref=\"504916917\" /><nd ref=\"504916916\" /><nd ref=\"504916915\" /><nd ref=\"504916914\" /><nd ref=\"504916913\" /><nd ref=\"2889672613\" /><nd ref=\"504916911\" /><nd ref=\"504916939\" /><nd ref=\"504916937\" /><nd ref=\"504916935\" /><nd ref=\"504916934\" /><nd ref=\"504916930\" /><nd ref=\"2889672609\" /><nd ref=\"504916932\" /><nd ref=\"504916928\" /><nd ref=\"504916927\" /><nd ref=\"504916926\" /><tag k=\"addr:city\" v=\"Seattle\" /><tag k=\"addr:housenumber\" v=\"915\" /><tag k=\"addr:postcode\" v=\"98107\" /><tag k=\"addr:street\" v=\"Northwest 45th Street\" /><tag k=\"building\" v=\"yes\" /><tag k=\"name\" v=\"Fred Meyer\" /><tag k=\"opening_hours\" v=\"Mo-Su 07:00-23:00\" /><tag k=\"shop\" v=\"garden_centre\" /><tag k=\"source\" v=\"King County GIS\" /></way><way id=\"42989264\" version=\"2\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:42Z\"><nd ref=\"504911068\" /><nd ref=\"538422731\" /><nd ref=\"538422735\" /><nd ref=\"538422733\" /><nd ref=\"538422732\" /><nd ref=\"538422743\" /><nd ref=\"538422744\" /><nd ref=\"538422741\" /><nd ref=\"538422749\" /><nd ref=\"538422747\" /><nd ref=\"538422800\" /><nd ref=\"538422798\" /><nd ref=\"2889672631\" /><nd ref=\"538422783\" /><tag k=\"highway\" v=\"service\" /><tag k=\"service\" v=\"parking_aisle\" /><tag k=\"surface\" v=\"asphalt\" /></way><way id=\"42989270\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:57Z\"><nd ref=\"538422757\" /><nd ref=\"538422732\" /><tag k=\"highway\" v=\"service\" /><tag k=\"oneway\" v=\"yes\" /><tag k=\"service\" v=\"parking_aisle\" /><tag k=\"surface\" v=\"asphalt\" /></way><way id=\"42989271\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:57Z\"><nd ref=\"538422733\" /><nd ref=\"538422759\" /><tag k=\"highway\" v=\"service\" /><tag k=\"oneway\" v=\"yes\" /><tag k=\"service\" v=\"parking_aisle\" /><tag k=\"surface\" v=\"asphalt\" /></way><way id=\"42989272\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:57Z\"><nd ref=\"538422832\" /><nd ref=\"538422735\" /><tag k=\"highway\" v=\"service\" /><tag k=\"oneway\" v=\"yes\" /><tag k=\"service\" v=\"parking_aisle\" /><tag k=\"surface\" v=\"asphalt\" /></way><way id=\"42989274\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:57Z\"><nd ref=\"538422850\" /><nd ref=\"538422851\" /><nd ref=\"538422844\" /><nd ref=\"538422839\" /><nd ref=\"538422841\" /><nd ref=\"538422845\" /><nd ref=\"504911072\" /><tag k=\"highway\" v=\"service\" /><tag k=\"service\" v=\"parking_aisle\" /><tag k=\"surface\" v=\"asphalt\" /></way><way id=\"42989275\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:58Z\"><nd ref=\"538422844\" /><nd ref=\"538422843\" /><nd ref=\"538422841\" /><tag k=\"highway\" v=\"service\" /><tag k=\"oneway\" v=\"yes\" /><tag k=\"service\" v=\"parking_aisle\" /><tag k=\"surface\" v=\"asphalt\" /></way><way id=\"42989276\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:58Z\"><nd ref=\"538422845\" /><nd ref=\"538422851\" /><tag k=\"highway\" v=\"service\" /><tag k=\"oneway\" v=\"yes\" /><tag k=\"service\" v=\"parking_aisle\" /><tag k=\"surface\" v=\"asphalt\" /></way><way id=\"44669179\" version=\"4\" changeset=\"24667727\" timestamp=\"2014-08-11T02:20:13Z\"><nd ref=\"566897505\" /><nd ref=\"566897520\" /><nd ref=\"566897518\" /><nd ref=\"566897517\" /><nd ref=\"566897516\" /><nd ref=\"566897511\" /><nd ref=\"566897509\" /><nd ref=\"566897507\" /><nd ref=\"566897505\" /><tag k=\"brewery\" v=\"yes\" /><tag k=\"building\" v=\"yes\" /><tag k=\"food\" v=\"yes\" /><tag k=\"microbrewery\" v=\"yes\" /><tag k=\"name\" v=\"Hale's Ales\" /></way><way id=\"53862223\" version=\"2\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:40Z\"><nd ref=\"680023522\" /><nd ref=\"2889669694\" /><nd ref=\"538422839\" /><tag k=\"highway\" v=\"service\" /><tag k=\"service\" v=\"driveway\" /></way><way id=\"158781278\" version=\"3\" changeset=\"18147994\" timestamp=\"2013-10-02T17:32:21Z\"><nd ref=\"53179274\" /><nd ref=\"1752970233\" /><nd ref=\"53189011\" /><nd ref=\"53194837\" /><nd ref=\"53194836\" /><nd ref=\"53095340\" /><nd ref=\"53112770\" /><nd ref=\"53194835\" /><nd ref=\"566896207\" /><nd ref=\"1709148214\" /><tag k=\"highway\" v=\"residential\" /><tag k=\"name\" v=\"Northwest 43rd Street\" /><tag k=\"tiger:cfcc\" v=\"A41\" /><tag k=\"tiger:county\" v=\"King, WA\" /><tag k=\"tiger:name_base\" v=\"43rd\" /><tag k=\"tiger:name_direction_prefix\" v=\"NW\" /><tag k=\"tiger:name_type\" v=\"St\" /></way><way id=\"160263609\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:03Z\"><nd ref=\"1723180942\" /><nd ref=\"1723180943\" /><nd ref=\"1723180986\" /><nd ref=\"1723180953\" /><nd ref=\"1723180942\" /><tag k=\"building\" v=\"yes\" /></way><way id=\"160263610\" version=\"2\" changeset=\"14866701\" timestamp=\"2013-02-01T03:20:09Z\"><nd ref=\"1723181006\" /><nd ref=\"1723180993\" /><nd ref=\"1723180934\" /><nd ref=\"1723181014\" /><nd ref=\"1723180925\" /><nd ref=\"1723181012\" /><nd ref=\"1723180990\" /><nd ref=\"1723181008\" /><nd ref=\"1723180977\" /><nd ref=\"1723180932\" /><nd ref=\"1723181006\" /><tag k=\"addr:city\" v=\"Seattle\" /><tag k=\"addr:housenumber\" v=\"805\" /><tag k=\"addr:postcode\" v=\"98107\" /><tag k=\"addr:street\" v=\"Northwest 42nd Street\" /><tag k=\"building\" v=\"yes\" /><tag k=\"source\" v=\"King County GIS\" /></way><way id=\"160263611\" version=\"3\" changeset=\"25279079\" timestamp=\"2014-09-07T04:03:19Z\"><nd ref=\"1723181010\" /><nd ref=\"1723180956\" /><nd ref=\"1723180997\" /><nd ref=\"1723180979\" /><nd ref=\"1723181010\" /><tag k=\"addr:city\" v=\"Seattle\" /><tag k=\"addr:housenumber\" v=\"4259\" /><tag k=\"addr:postcode\" v=\"98107\" /><tag k=\"addr:street\" v=\"8th Avenue Northwest\" /><tag k=\"building\" v=\"yes\" /><tag k=\"name\" v=\"Albert Lee Distribution Center\" /><tag k=\"shop\" v=\"appliance\" /><tag k=\"source\" v=\"King County GIS\" /><tag k=\"website\" v=\"http://www.albertleeappliance.com/\" /></way><way id=\"160263613\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:03Z\"><nd ref=\"1723180941\" /><nd ref=\"1723180974\" /><nd ref=\"1723180937\" /><nd ref=\"1723180929\" /><nd ref=\"1723180999\" /><nd ref=\"1723180984\" /><nd ref=\"1723180948\" /><nd ref=\"1723180941\" /><tag k=\"building\" v=\"yes\" /></way><way id=\"160263614\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:03Z\"><nd ref=\"1723180939\" /><nd ref=\"1723180975\" /><nd ref=\"1723180995\" /><nd ref=\"1723180936\" /><nd ref=\"1723180910\" /><nd ref=\"1723180909\" /><nd ref=\"1723180949\" /><nd ref=\"1723180946\" /><nd ref=\"1723180939\" /><tag k=\"building\" v=\"yes\" /></way><way id=\"213295265\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:50:16Z\"><nd ref=\"2230240938\" /><nd ref=\"2230240940\" /><nd ref=\"2230241113\" /><nd ref=\"2230241110\" /><nd ref=\"2230240938\" /><tag k=\"building\" v=\"yes\" /><tag k=\"source\" v=\"data.seattle.gov\" /></way><way id=\"213295301\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:50:18Z\"><nd ref=\"2230239177\" /><nd ref=\"2230239218\" /><nd ref=\"2230239369\" /><nd ref=\"2230239294\" /><nd ref=\"2230239177\" /><tag k=\"building\" v=\"yes\" /><tag k=\"source\" v=\"data.seattle.gov\" /></way><way id=\"213295404\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:50:23Z\"><nd ref=\"2230239566\" /><nd ref=\"2230239561\" /><nd ref=\"2230239532\" /><nd ref=\"2230239401\" /><nd ref=\"2230239404\" /><nd ref=\"2230239383\" /><nd ref=\"2230239385\" /><nd ref=\"2230239566\" /><tag k=\"building\" v=\"yes\" /><tag k=\"source\" v=\"data.seattle.gov\" /></way><way id=\"213295522\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:50:29Z\"><nd ref=\"2230239211\" /><nd ref=\"2230239208\" /><nd ref=\"2230239096\" /><nd ref=\"2230239101\" /><nd ref=\"2230239023\" /><nd ref=\"2230239026\" /><nd ref=\"2230239137\" /><nd ref=\"2230239140\" /><nd ref=\"2230239211\" /><tag k=\"building\" v=\"yes\" /><tag k=\"source\" v=\"data.seattle.gov\" /></way><way id=\"217499409\" version=\"1\" changeset=\"15731010\" timestamp=\"2013-04-14T22:37:23Z\"><nd ref=\"2267748406\" /><nd ref=\"2267748400\" /><nd ref=\"2267748404\" /><nd ref=\"2267748407\" /><nd ref=\"2267748406\" /><tag k=\"building\" v=\"yes\" /><tag k=\"source\" v=\"data.seattle.gov\" /></way><way id=\"217499460\" version=\"1\" changeset=\"15731010\" timestamp=\"2013-04-14T22:37:27Z\"><nd ref=\"2267748422\" /><nd ref=\"2267748411\" /><nd ref=\"2267748415\" /><nd ref=\"2267748424\" /><nd ref=\"2267748422\" /><tag k=\"building\" v=\"yes\" /><tag k=\"source\" v=\"data.seattle.gov\" /></way><way id=\"233133914\" version=\"1\" changeset=\"17304993\" timestamp=\"2013-08-11T15:38:12Z\"><nd ref=\"2414679852\" /><nd ref=\"2414679859\" /><nd ref=\"2414679860\" /><nd ref=\"2414679862\" /><nd ref=\"2414679863\" /><nd ref=\"2414679861\" /><nd ref=\"2414679858\" /><nd ref=\"2414679857\" /><nd ref=\"2414679856\" /><nd ref=\"2414679855\" /><nd ref=\"2414679854\" /><nd ref=\"2414679853\" /><nd ref=\"2414679852\" /><tag k=\"area\" v=\"yes\" /><tag k=\"building\" v=\"yes\" /></way><way id=\"234714284\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:57Z\"><nd ref=\"2428608457\" /><nd ref=\"2428608456\" /><nd ref=\"2428608462\" /><nd ref=\"2428608463\" /><nd ref=\"2428608464\" /><nd ref=\"2428608465\" /><nd ref=\"2428608460\" /><nd ref=\"2428608461\" /><nd ref=\"2428608459\" /><nd ref=\"2428608458\" /><nd ref=\"2428608457\" /><tag k=\"building\" v=\"yes\" /><tag k=\"source\" v=\"data.seattle.gov\" /></way><way id=\"234714287\" version=\"3\" changeset=\"28009821\" timestamp=\"2015-01-09T00:16:30Z\"><nd ref=\"2428608453\" /><nd ref=\"2428608437\" /><nd ref=\"2428608438\" /><nd ref=\"2428608448\" /><nd ref=\"2428608450\" /><nd ref=\"2428608451\" /><nd ref=\"2428608452\" /><nd ref=\"2428608447\" /><nd ref=\"2428608449\" /><nd ref=\"2428608455\" /><nd ref=\"2428608454\" /><nd ref=\"2428608453\" /><tag k=\"amenity\" v=\"pub\" /><tag k=\"building\" v=\"yes\" /><tag k=\"name\" v=\"Four B's\" /><tag k=\"source\" v=\"data.seattle.gov\" /></way><way id=\"234714294\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:57Z\"><nd ref=\"2428608481\" /><nd ref=\"2428608480\" /><nd ref=\"2428608475\" /><nd ref=\"2428608474\" /><nd ref=\"2428608472\" /><nd ref=\"2428608473\" /><nd ref=\"2428608468\" /><nd ref=\"2428608469\" /><nd ref=\"2428608476\" /><nd ref=\"2428608477\" /><nd ref=\"2428608479\" /><nd ref=\"2428608478\" /><nd ref=\"2428608481\" /><tag k=\"building\" v=\"yes\" /><tag k=\"source\" v=\"data.seattle.gov\" /></way><way id=\"234714295\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:58Z\"><nd ref=\"2428608490\" /><nd ref=\"2428608506\" /><nd ref=\"2428608504\" /><nd ref=\"2428608494\" /><nd ref=\"2428608493\" /><nd ref=\"2428608492\" /><nd ref=\"2428608491\" /><nd ref=\"2428608487\" /><nd ref=\"2428608488\" /><nd ref=\"2428608485\" /><nd ref=\"2428608486\" /><nd ref=\"2428608482\" /><nd ref=\"2428608483\" /><nd ref=\"2428608489\" /><nd ref=\"2428608490\" /><tag k=\"building\" v=\"yes\" /><tag k=\"source\" v=\"data.seattle.gov\" /></way><way id=\"234714310\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:58Z\"><nd ref=\"2428608619\" /><nd ref=\"2428608617\" /><nd ref=\"2428608594\" /><nd ref=\"2428608596\" /><nd ref=\"2428608592\" /><nd ref=\"2428608590\" /><nd ref=\"2428608565\" /><nd ref=\"2428608568\" /><nd ref=\"2428608557\" /><nd ref=\"2428608555\" /><nd ref=\"2428608534\" /><nd ref=\"2428608515\" /><nd ref=\"2428608516\" /><nd ref=\"2428608508\" /><nd ref=\"2428608510\" /><nd ref=\"2428608511\" /><nd ref=\"2428608513\" /><nd ref=\"2428608552\" /><nd ref=\"2428608550\" /><nd ref=\"2428608580\" /><nd ref=\"2428608577\" /><nd ref=\"2428608587\" /><nd ref=\"2428608584\" /><nd ref=\"2428608603\" /><nd ref=\"2428608607\" /><nd ref=\"2428608605\" /><nd ref=\"2428608609\" /><nd ref=\"2428608611\" /><nd ref=\"2428608623\" /><nd ref=\"2428608621\" /><nd ref=\"2428608619\" /><tag k=\"building\" v=\"yes\" /><tag k=\"source\" v=\"data.seattle.gov\" /></way><way id=\"234714311\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:58Z\"><nd ref=\"2428608502\" /><nd ref=\"2428608471\" /><nd ref=\"2428608470\" /><nd ref=\"2428608500\" /><nd ref=\"2428608502\" /><tag k=\"building\" v=\"yes\" /><tag k=\"source\" v=\"data.seattle.gov\" /></way><way id=\"234714317\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:59Z\"><nd ref=\"2428608497\" /><nd ref=\"2428608467\" /><nd ref=\"2428608466\" /><nd ref=\"2428608484\" /><nd ref=\"2428608495\" /><nd ref=\"2428608498\" /><nd ref=\"2428608499\" /><nd ref=\"2428608496\" /><nd ref=\"2428608497\" /><tag k=\"building\" v=\"yes\" /><tag k=\"source\" v=\"data.seattle.gov\" /></way><way id=\"285293320\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:35Z\"><nd ref=\"2889669699\" /><nd ref=\"2889669698\" /><nd ref=\"2889669697\" /><nd ref=\"2889669696\" /><nd ref=\"2889669695\" /><nd ref=\"2889669694\" /><tag k=\"highway\" v=\"path\" /></way><way id=\"285293321\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:35Z\"><nd ref=\"2889669700\" /><nd ref=\"2889669697\" /><tag k=\"highway\" v=\"steps\" /></way><way id=\"285293322\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:36Z\"><nd ref=\"2889669695\" /><nd ref=\"2889669700\" /><tag k=\"highway\" v=\"path\" /></way><way id=\"285293323\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:36Z\"><nd ref=\"2889672608\" /><nd ref=\"2889672607\" /><nd ref=\"2889672606\" /><nd ref=\"2889672605\" /><nd ref=\"2889672604\" /><nd ref=\"2889672603\" /><nd ref=\"2889672602\" /><nd ref=\"2889672601\" /><tag k=\"highway\" v=\"footway\" /></way><way id=\"285293324\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:36Z\"><nd ref=\"2889672609\" /><nd ref=\"2889669696\" /><tag k=\"highway\" v=\"path\" /></way><way id=\"285293325\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:36Z\"><nd ref=\"2889669693\" /><nd ref=\"2889672610\" /><nd ref=\"2889672629\" /><nd ref=\"2889672609\" /><tag k=\"highway\" v=\"path\" /><tag k=\"tunnel\" v=\"yes\" /></way><way id=\"285293326\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:36Z\"><nd ref=\"2889672618\" /><nd ref=\"2889672617\" /><nd ref=\"2889672616\" /><nd ref=\"2889672615\" /><nd ref=\"2889672614\" /><nd ref=\"2889672613\" /><nd ref=\"2889672612\" /><nd ref=\"2889672611\" /><tag k=\"highway\" v=\"service\" /></way><way id=\"285293329\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:36Z\"><nd ref=\"2889672629\" /><nd ref=\"2889672628\" /><nd ref=\"2889672627\" /><tag k=\"highway\" v=\"path\" /><tag k=\"tunnel\" v=\"yes\" /></way><way id=\"285293332\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:36Z\"><nd ref=\"2889669698\" /><nd ref=\"2889672638\" /><nd ref=\"2889672627\" /><nd ref=\"2889672634\" /><tag k=\"highway\" v=\"path\" /></way><way id=\"305197030\" version=\"3\" changeset=\"27281941\" timestamp=\"2014-12-06T02:57:11Z\"><nd ref=\"3155658710\" /><nd ref=\"3096003526\" /><nd ref=\"1752970227\" /><nd ref=\"3155658709\" /><nd ref=\"3155658708\" /><nd ref=\"3096003521\" /><nd ref=\"3155658707\" /><nd ref=\"3096003520\" /><nd ref=\"3096003519\" /><nd ref=\"67569701\" /><nd ref=\"2644688332\" /><nd ref=\"67569707\" /><nd ref=\"67569713\" /><nd ref=\"1752970235\" /><nd ref=\"2644688331\" /><nd ref=\"1752970237\" /><nd ref=\"60979342\" /><nd ref=\"2889672601\" /><nd ref=\"1752970239\" /><nd ref=\"29937458\" /><nd ref=\"29937457\" /><nd ref=\"29937456\" /><nd ref=\"29937454\" /><tag k=\"bicycle\" v=\"designated\" /><tag k=\"foot\" v=\"yes\" /><tag k=\"highway\" v=\"cycleway\" /><tag k=\"name\" v=\"Burke-Gilman Trail\" /><tag k=\"segregated\" v=\"no\" /><tag k=\"source\" v=\"Local Knwoladge\" /><tag k=\"surface\" v=\"asphalt\" /></way></osm>";
    private static final String BALLARD_NODES_XML = "<?xml version='1.0' encoding='UTF-8' ?><osm version=\"0.6\" generator=\"OpenMapKit 0.1\" user=\"theoutpost\"><node id=\"504911053\" version=\"1\" changeset=\"2565517\" timestamp=\"2009-09-22T07:08:19Z\" lat=\"47.6608743\" lon=\"-122.3705468\" /><node id=\"504911054\" version=\"1\" changeset=\"2565517\" timestamp=\"2009-09-22T07:08:19Z\" lat=\"47.6608863\" lon=\"-122.3705885\" /><node id=\"504911056\" version=\"1\" changeset=\"2565517\" timestamp=\"2009-09-22T07:08:19Z\" lat=\"47.660493\" lon=\"-122.3696468\" /><node id=\"504911057\" version=\"1\" changeset=\"2565517\" timestamp=\"2009-09-22T07:08:19Z\" lat=\"47.6605371\" lon=\"-122.3697005\" /><node id=\"504916914\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.6598962\" lon=\"-122.3689237\" /><node id=\"504916916\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.660374\" lon=\"-122.3692869\" /><node id=\"504916917\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.66036\" lon=\"-122.369315\" /><node id=\"504916919\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.6604916\" lon=\"-122.3694599\" /><node id=\"504916920\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.6605104\" lon=\"-122.3694222\" /><node id=\"504916922\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.6606315\" lon=\"-122.3695553\" /><node id=\"504916924\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.6607279\" lon=\"-122.3693622\" /><node id=\"504916925\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.6607886\" lon=\"-122.369429\" /><node id=\"538422732\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:52Z\" lat=\"47.6593051\" lon=\"-122.369225\" /><node id=\"538422733\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:52Z\" lat=\"47.659185\" lon=\"-122.3690411\" /><node id=\"538422735\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:52Z\" lat=\"47.6590938\" lon=\"-122.3688999\" /><node id=\"538422741\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:52Z\" lat=\"47.6595401\" lon=\"-122.3696103\" /><node id=\"538422743\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:52Z\" lat=\"47.6594014\" lon=\"-122.3693719\" /><node id=\"538422744\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:52Z\" lat=\"47.6594108\" lon=\"-122.3693881\" /><node id=\"538422747\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:52Z\" lat=\"47.6599292\" lon=\"-122.3696223\" /><node id=\"538422749\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:52Z\" lat=\"47.659725\" lon=\"-122.369616\" /><node id=\"538422777\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:53Z\" lat=\"47.6599729\" lon=\"-122.3690833\" /><node id=\"538422779\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:53Z\" lat=\"47.6598531\" lon=\"-122.3689535\" /><node id=\"538422783\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:53Z\" lat=\"47.6604429\" lon=\"-122.3698925\" /><node id=\"538422791\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:53Z\" lat=\"47.6608574\" lon=\"-122.3705045\" /><node id=\"538422798\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:54Z\" lat=\"47.6602169\" lon=\"-122.3696312\" /><node id=\"538422799\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:54Z\" lat=\"47.6602345\" lon=\"-122.3693667\" /><node id=\"538422800\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:54Z\" lat=\"47.6601065\" lon=\"-122.3696278\" /><node id=\"538422801\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:54Z\" lat=\"47.6601196\" lon=\"-122.3692423\" /><node id=\"2889669693\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6606937\" lon=\"-122.3694307\" /><node id=\"2889672610\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6606706\" lon=\"-122.3693551\" /><node id=\"2889672618\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6598108\" lon=\"-122.3689077\" /><node id=\"2889672626\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6597947\" lon=\"-122.3688902\" /><node id=\"2889672631\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:35Z\" lat=\"47.6604077\" lon=\"-122.3698518\" /><node id=\"2889672632\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:35Z\" lat=\"47.6605044\" lon=\"-122.3696606\" /><node id=\"2889672633\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:35Z\" lat=\"47.6605809\" lon=\"-122.3694997\" /><node id=\"2889672634\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:35Z\" lat=\"47.6605729\" lon=\"-122.3695165\" /><node id=\"504916926\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.661092\" lon=\"-122.3688212\" /><node id=\"53149116\" version=\"7\" changeset=\"6214568\" timestamp=\"2010-10-29T06:01:59Z\" lat=\"47.6541565\" lon=\"-122.3600558\" /><node id=\"53197489\" version=\"5\" changeset=\"3341821\" timestamp=\"2009-12-10T14:23:02Z\" lat=\"47.6532822\" lon=\"-122.3590767\" /><node id=\"67569764\" version=\"3\" changeset=\"326961\" timestamp=\"2007-10-27T00:50:55Z\" lat=\"47.6533988\" lon=\"-122.3592865\"><tag k=\"created_by\" v=\"JOSM\" /></node><node id=\"67569766\" version=\"3\" changeset=\"326961\" timestamp=\"2007-10-27T00:50:55Z\" lat=\"47.6534945\" lon=\"-122.3594287\"><tag k=\"created_by\" v=\"JOSM\" /></node><node id=\"67569769\" version=\"3\" changeset=\"326961\" timestamp=\"2007-10-27T00:50:57Z\" lat=\"47.6536082\" lon=\"-122.359566\"><tag k=\"created_by\" v=\"JOSM\" /></node><node id=\"1752970231\" version=\"1\" changeset=\"11599644\" timestamp=\"2012-05-14T20:20:13Z\" lat=\"47.6588346\" lon=\"-122.3661194\" /><node id=\"29937458\" version=\"12\" changeset=\"11599644\" timestamp=\"2012-05-14T20:21:12Z\" lat=\"47.660959\" lon=\"-122.3679437\" /><node id=\"53095338\" version=\"6\" changeset=\"6214568\" timestamp=\"2010-10-29T06:01:58Z\" lat=\"47.6577845\" lon=\"-122.3629059\" /><node id=\"53095340\" version=\"6\" changeset=\"6214568\" timestamp=\"2010-10-29T06:02:00Z\" lat=\"47.658949\" lon=\"-122.362917\" /><node id=\"53095342\" version=\"5\" changeset=\"2228009\" timestamp=\"2009-08-21T21:03:24Z\" lat=\"47.6599952\" lon=\"-122.3629095\" /><node id=\"53106426\" version=\"6\" changeset=\"6214568\" timestamp=\"2010-10-29T06:02:01Z\" lat=\"47.6569308\" lon=\"-122.3627305\" /><node id=\"53179273\" version=\"6\" changeset=\"3341821\" timestamp=\"2009-12-10T14:09:55Z\" lat=\"47.6585229\" lon=\"-122.366138\" /><node id=\"53179276\" version=\"5\" changeset=\"3341821\" timestamp=\"2009-12-10T14:09:55Z\" lat=\"47.6599752\" lon=\"-122.3661191\" /><node id=\"53179278\" version=\"6\" changeset=\"5012635\" timestamp=\"2010-06-17T21:39:46Z\" lat=\"47.6606671\" lon=\"-122.366404\"><tag k=\"highway\" v=\"traffic_signals\" /></node><node id=\"53183015\" version=\"5\" changeset=\"3341821\" timestamp=\"2009-12-10T14:11:24Z\" lat=\"47.6585222\" lon=\"-122.3677302\" /><node id=\"53189008\" version=\"7\" changeset=\"11599644\" timestamp=\"2012-05-14T20:21:33Z\" lat=\"47.6577728\" lon=\"-122.3650606\" /><node id=\"53189011\" version=\"8\" changeset=\"11599644\" timestamp=\"2012-05-14T20:21:33Z\" lat=\"47.6589491\" lon=\"-122.365064\" /><node id=\"53189540\" version=\"6\" changeset=\"6214568\" timestamp=\"2010-10-29T06:01:55Z\" lat=\"47.6577877\" lon=\"-122.3635782\" /><node id=\"53189541\" version=\"7\" changeset=\"11599644\" timestamp=\"2012-05-14T20:21:34Z\" lat=\"47.6577694\" lon=\"-122.3640013\" /><node id=\"53189547\" version=\"4\" changeset=\"3341821\" timestamp=\"2009-12-10T14:17:16Z\" lat=\"47.6577707\" lon=\"-122.3672121\" /><node id=\"53194836\" version=\"6\" changeset=\"5012635\" timestamp=\"2010-06-17T21:39:32Z\" lat=\"47.6589555\" lon=\"-122.3639936\" /><node id=\"53194837\" version=\"7\" changeset=\"6214568\" timestamp=\"2010-10-29T06:01:58Z\" lat=\"47.6589634\" lon=\"-122.3647357\"><tag k=\"highway\" v=\"traffic_signals\" /></node><node id=\"53197113\" version=\"5\" changeset=\"3341821\" timestamp=\"2009-12-10T14:22:48Z\" lat=\"47.6600023\" lon=\"-122.3639987\" /><node id=\"53197114\" version=\"5\" changeset=\"2757522\" timestamp=\"2009-10-06T07:41:48Z\" lat=\"47.6600136\" lon=\"-122.3657472\" /><node id=\"53197491\" version=\"5\" changeset=\"2757522\" timestamp=\"2009-10-06T07:41:54Z\" lat=\"47.6582444\" lon=\"-122.3640069\" /><node id=\"53243384\" version=\"5\" changeset=\"3421031\" timestamp=\"2009-12-21T15:31:04Z\" lat=\"47.6576231\" lon=\"-122.3671272\" /><node id=\"67569717\" version=\"3\" changeset=\"326961\" timestamp=\"2007-10-27T00:57:38Z\" lat=\"47.6599061\" lon=\"-122.3661794\"><tag k=\"created_by\" v=\"JOSM\" /></node><node id=\"67569725\" version=\"3\" changeset=\"326961\" timestamp=\"2007-10-27T00:57:38Z\" lat=\"47.6599935\" lon=\"-122.366021\"><tag k=\"created_by\" v=\"JOSM\" /></node><node id=\"67569743\" version=\"3\" changeset=\"326961\" timestamp=\"2007-10-27T00:50:59Z\" lat=\"47.6575714\" lon=\"-122.3633953\"><tag k=\"created_by\" v=\"JOSM\" /></node><node id=\"504911052\" version=\"1\" changeset=\"2565517\" timestamp=\"2009-09-22T07:08:19Z\" lat=\"47.6595717\" lon=\"-122.3686486\" /><node id=\"504911067\" version=\"1\" changeset=\"2565517\" timestamp=\"2009-09-22T07:08:20Z\" lat=\"47.6592606\" lon=\"-122.3680109\" /><node id=\"504911068\" version=\"1\" changeset=\"2565517\" timestamp=\"2009-09-22T07:08:20Z\" lat=\"47.6591944\" lon=\"-122.3681926\" /><node id=\"504911069\" version=\"1\" changeset=\"2565517\" timestamp=\"2009-09-22T07:08:20Z\" lat=\"47.6592593\" lon=\"-122.3667867\" /><node id=\"504911071\" version=\"1\" changeset=\"2565517\" timestamp=\"2009-09-22T07:08:20Z\" lat=\"47.6595798\" lon=\"-122.3667896\" /><node id=\"504911072\" version=\"2\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:58Z\" lat=\"47.6597752\" lon=\"-122.3669905\" /><node id=\"504911073\" version=\"1\" changeset=\"2565517\" timestamp=\"2009-09-22T07:08:20Z\" lat=\"47.6610329\" lon=\"-122.3683452\" /><node id=\"504916911\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.6596815\" lon=\"-122.3685254\" /><node id=\"504916913\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.6596293\" lon=\"-122.36863\" /><node id=\"504916915\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.6599486\" lon=\"-122.3688188\" /><node id=\"504916927\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.6610243\" lon=\"-122.3687466\" /><node id=\"504916930\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.6593454\" lon=\"-122.3681474\" /><node id=\"504916932\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.6598155\" lon=\"-122.3672055\" /><node id=\"504916934\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.6594667\" lon=\"-122.3682809\" /><node id=\"504916935\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.6594499\" lon=\"-122.3683147\" /><node id=\"504916937\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:06Z\" lat=\"47.6595713\" lon=\"-122.3684483\" /><node id=\"504916939\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:06Z\" lat=\"47.6595855\" lon=\"-122.3684198\" /><node id=\"538422731\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:52Z\" lat=\"47.6589287\" lon=\"-122.3686492\" /><node id=\"538422755\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:52Z\" lat=\"47.6597071\" lon=\"-122.3687953\" /><node id=\"538422757\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:52Z\" lat=\"47.6595867\" lon=\"-122.3686648\" /><node id=\"538422759\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:52Z\" lat=\"47.6594495\" lon=\"-122.3685009\" /><node id=\"538422832\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:55Z\" lat=\"47.659349\" lon=\"-122.3683794\" /><node id=\"538422839\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:55Z\" lat=\"47.6594261\" lon=\"-122.367721\" /><node id=\"538422841\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:55Z\" lat=\"47.6595923\" lon=\"-122.3673732\" /><node id=\"538422843\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:55Z\" lat=\"47.6594967\" lon=\"-122.3672824\" /><node id=\"538422844\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:55Z\" lat=\"47.6594322\" lon=\"-122.3672777\" /><node id=\"538422845\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:55Z\" lat=\"47.6596844\" lon=\"-122.3671804\" /><node id=\"538422850\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:55Z\" lat=\"47.6594389\" lon=\"-122.366789\" /><node id=\"538422851\" version=\"1\" changeset=\"2923553\" timestamp=\"2009-10-22T18:37:55Z\" lat=\"47.6594376\" lon=\"-122.3668796\" /><node id=\"566897505\" version=\"1\" changeset=\"3157057\" timestamp=\"2009-11-19T06:09:32Z\" lat=\"47.6590403\" lon=\"-122.3651827\" /><node id=\"566897507\" version=\"1\" changeset=\"3157057\" timestamp=\"2009-11-19T06:09:33Z\" lat=\"47.6592079\" lon=\"-122.3651827\" /><node id=\"566897509\" version=\"1\" changeset=\"3157057\" timestamp=\"2009-11-19T06:09:33Z\" lat=\"47.6594334\" lon=\"-122.3654316\" /><node id=\"566897511\" version=\"1\" changeset=\"3157057\" timestamp=\"2009-11-19T06:09:33Z\" lat=\"47.6594449\" lon=\"-122.3660153\" /><node id=\"566897516\" version=\"1\" changeset=\"3157057\" timestamp=\"2009-11-19T06:09:33Z\" lat=\"47.6592946\" lon=\"-122.3659895\" /><node id=\"566897517\" version=\"1\" changeset=\"3157057\" timestamp=\"2009-11-19T06:09:34Z\" lat=\"47.6592946\" lon=\"-122.365938\" /><node id=\"566897518\" version=\"1\" changeset=\"3157057\" timestamp=\"2009-11-19T06:09:34Z\" lat=\"47.6591154\" lon=\"-122.3659295\" /><node id=\"566897520\" version=\"1\" changeset=\"3157057\" timestamp=\"2009-11-19T06:09:34Z\" lat=\"47.6590518\" lon=\"-122.3658436\" /><node id=\"680023522\" version=\"1\" changeset=\"4296062\" timestamp=\"2010-04-01T17:33:05Z\" lat=\"47.6592603\" lon=\"-122.3677218\" /><node id=\"1723180909\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:00Z\" lat=\"47.6582807\" lon=\"-122.3647033\" /><node id=\"1723180910\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:00Z\" lat=\"47.6582601\" lon=\"-122.3647656\" /><node id=\"1723180925\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:00Z\" lat=\"47.6575797\" lon=\"-122.3662\" /><node id=\"1723180929\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:00Z\" lat=\"47.6580686\" lon=\"-122.3649104\" /><node id=\"1723180932\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:00Z\" lat=\"47.6577243\" lon=\"-122.3660605\" /><node id=\"1723180934\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:00Z\" lat=\"47.6576267\" lon=\"-122.3669993\" /><node id=\"1723180936\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:00Z\" lat=\"47.6583096\" lon=\"-122.3648106\" /><node id=\"1723180937\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:01Z\" lat=\"47.6581506\" lon=\"-122.3649233\" /><node id=\"1723180939\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:01Z\" lat=\"47.6584903\" lon=\"-122.3645692\" /><node id=\"1723180941\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:01Z\" lat=\"47.6582301\" lon=\"-122.3644995\" /><node id=\"1723180942\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:01Z\" lat=\"47.6587709\" lon=\"-122.3684703\" /><node id=\"1723180943\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:01Z\" lat=\"47.6575011\" lon=\"-122.3677069\" /><node id=\"1723180946\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:01Z\" lat=\"47.6583179\" lon=\"-122.3643954\" /><node id=\"1723180948\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:01Z\" lat=\"47.6582337\" lon=\"-122.3642581\" /><node id=\"1723180949\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:01Z\" lat=\"47.6582168\" lon=\"-122.3646583\" /><node id=\"1723180953\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:01Z\" lat=\"47.6588871\" lon=\"-122.3680443\" /><node id=\"1723180956\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:01Z\" lat=\"47.658714\" lon=\"-122.3677157\" /><node id=\"1723180974\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:01Z\" lat=\"47.6581543\" lon=\"-122.3645049\" /><node id=\"1723180975\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:01Z\" lat=\"47.6583891\" lon=\"-122.3648321\" /><node id=\"1723180977\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:01Z\" lat=\"47.6576845\" lon=\"-122.3659747\" /><node id=\"1723180979\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:01Z\" lat=\"47.6589576\" lon=\"-122.3662465\" /><node id=\"1723180984\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:02Z\" lat=\"47.6578652\" lon=\"-122.3642527\" /><node id=\"1723180986\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:02Z\" lat=\"47.6576172\" lon=\"-122.367281\" /><node id=\"1723180990\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:02Z\" lat=\"47.6573485\" lon=\"-122.3660284\" /><node id=\"1723180993\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:02Z\" lat=\"47.6576303\" lon=\"-122.3670798\" /><node id=\"1723180995\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:02Z\" lat=\"47.6583313\" lon=\"-122.3647677\" /><node id=\"1723180997\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:02Z\" lat=\"47.6587191\" lon=\"-122.3662446\" /><node id=\"1723180999\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:02Z\" lat=\"47.6578591\" lon=\"-122.3647119\" /><node id=\"1723181006\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:02Z\" lat=\"47.6577098\" lon=\"-122.367069\" /><node id=\"1723181008\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:02Z\" lat=\"47.6574244\" lon=\"-122.3659693\" /><node id=\"1723181010\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:02Z\" lat=\"47.6589525\" lon=\"-122.3677175\" /><node id=\"1723181012\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:02Z\" lat=\"47.6573521\" lon=\"-122.3662\" /><node id=\"1723181014\" version=\"1\" changeset=\"11356766\" timestamp=\"2012-04-19T18:52:02Z\" lat=\"47.657587\" lon=\"-122.3669939\" /><node id=\"2230239023\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:13Z\" lat=\"47.6573036\" lon=\"-122.367344\" /><node id=\"2230239026\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:13Z\" lat=\"47.6573037\" lon=\"-122.3669898\" /><node id=\"2230239096\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:13Z\" lat=\"47.6573573\" lon=\"-122.3675953\" /><node id=\"2230239101\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:13Z\" lat=\"47.6573574\" lon=\"-122.367344\" /><node id=\"2230239137\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:14Z\" lat=\"47.6573933\" lon=\"-122.3669898\" /><node id=\"2230239140\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:14Z\" lat=\"47.6573933\" lon=\"-122.3670893\" /><node id=\"2230239177\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:14Z\" lat=\"47.657446\" lon=\"-122.3647192\" /><node id=\"2230239208\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:14Z\" lat=\"47.6574775\" lon=\"-122.3675954\" /><node id=\"2230239211\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:14Z\" lat=\"47.6574776\" lon=\"-122.3670894\" /><node id=\"2230239218\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:14Z\" lat=\"47.6574829\" lon=\"-122.3646633\" /><node id=\"2230239294\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:15Z\" lat=\"47.657517\" lon=\"-122.3648219\" /><node id=\"2230239369\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:15Z\" lat=\"47.6575539\" lon=\"-122.3647659\" /><node id=\"2230239383\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:16Z\" lat=\"47.6575681\" lon=\"-122.3643213\" /><node id=\"2230239385\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:16Z\" lat=\"47.6575701\" lon=\"-122.3641569\" /><node id=\"2230239401\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:16Z\" lat=\"47.6575893\" lon=\"-122.3644778\" /><node id=\"2230239404\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:16Z\" lat=\"47.6575912\" lon=\"-122.3643219\" /><node id=\"2230239532\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:17Z\" lat=\"47.657662\" lon=\"-122.3645472\" /><node id=\"2230239561\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:17Z\" lat=\"47.6576888\" lon=\"-122.3645479\" /><node id=\"2230239566\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:17Z\" lat=\"47.6576934\" lon=\"-122.3641601\" /><node id=\"2230240938\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:28Z\" lat=\"47.6587804\" lon=\"-122.3643262\" /><node id=\"2230240940\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:28Z\" lat=\"47.6587823\" lon=\"-122.364138\" /><node id=\"2230241110\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:30Z\" lat=\"47.6588604\" lon=\"-122.364328\" /><node id=\"2230241113\" version=\"1\" changeset=\"15522426\" timestamp=\"2013-03-28T02:49:30Z\" lat=\"47.6588623\" lon=\"-122.3641399\" /><node id=\"2267748400\" version=\"1\" changeset=\"15731010\" timestamp=\"2013-04-14T22:35:23Z\" lat=\"47.6599319\" lon=\"-122.3671295\" /><node id=\"2267748404\" version=\"1\" changeset=\"15731010\" timestamp=\"2013-04-14T22:35:23Z\" lat=\"47.6599563\" lon=\"-122.3670728\" /><node id=\"2267748406\" version=\"1\" changeset=\"15731010\" timestamp=\"2013-04-14T22:35:23Z\" lat=\"47.6600232\" lon=\"-122.3672158\" /><node id=\"2267748407\" version=\"1\" changeset=\"15731010\" timestamp=\"2013-04-14T22:35:23Z\" lat=\"47.6600476\" lon=\"-122.3671592\" /><node id=\"2267748411\" version=\"1\" changeset=\"15731010\" timestamp=\"2013-04-14T22:35:24Z\" lat=\"47.6600707\" lon=\"-122.3672604\" /><node id=\"2267748415\" version=\"1\" changeset=\"15731010\" timestamp=\"2013-04-14T22:35:24Z\" lat=\"47.6600965\" lon=\"-122.3672042\" /><node id=\"2267748422\" version=\"1\" changeset=\"15731010\" timestamp=\"2013-04-14T22:35:24Z\" lat=\"47.6601624\" lon=\"-122.3673526\" /><node id=\"2267748424\" version=\"1\" changeset=\"15731010\" timestamp=\"2013-04-14T22:35:24Z\" lat=\"47.6601882\" lon=\"-122.3672963\" /><node id=\"2414679852\" version=\"1\" changeset=\"17304993\" timestamp=\"2013-08-11T15:38:11Z\" lat=\"47.6584488\" lon=\"-122.3673527\" /><node id=\"2414679853\" version=\"1\" changeset=\"17304993\" timestamp=\"2013-08-11T15:38:11Z\" lat=\"47.6584553\" lon=\"-122.3657071\" /><node id=\"2414679854\" version=\"1\" changeset=\"17304993\" timestamp=\"2013-08-11T15:38:11Z\" lat=\"47.6578338\" lon=\"-122.3657016\" /><node id=\"2414679855\" version=\"1\" changeset=\"17304993\" timestamp=\"2013-08-11T15:38:11Z\" lat=\"47.6578314\" lon=\"-122.366319\" /><node id=\"2414679856\" version=\"1\" changeset=\"17304993\" timestamp=\"2013-08-11T15:38:11Z\" lat=\"47.6581196\" lon=\"-122.3663215\" /><node id=\"2414679857\" version=\"1\" changeset=\"17304993\" timestamp=\"2013-08-11T15:38:11Z\" lat=\"47.6581213\" lon=\"-122.3658944\" /><node id=\"2414679858\" version=\"1\" changeset=\"17304993\" timestamp=\"2013-08-11T15:38:11Z\" lat=\"47.658173\" lon=\"-122.3658948\" /><node id=\"2414679859\" version=\"1\" changeset=\"17304993\" timestamp=\"2013-08-11T15:38:11Z\" lat=\"47.658173\" lon=\"-122.3673503\" /><node id=\"2414679860\" version=\"1\" changeset=\"17304993\" timestamp=\"2013-08-11T15:38:11Z\" lat=\"47.6581738\" lon=\"-122.3671504\" /><node id=\"2414679861\" version=\"1\" changeset=\"17304993\" timestamp=\"2013-08-11T15:38:11Z\" lat=\"47.6581686\" lon=\"-122.3669988\" /><node id=\"2414679862\" version=\"1\" changeset=\"17304993\" timestamp=\"2013-08-11T15:38:11Z\" lat=\"47.6580211\" lon=\"-122.367149\" /><node id=\"2414679863\" version=\"1\" changeset=\"17304993\" timestamp=\"2013-08-11T15:38:11Z\" lat=\"47.6580217\" lon=\"-122.3669975\" /><node id=\"2428608437\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6590306\" lon=\"-122.3645546\" /><node id=\"2428608438\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.659031\" lon=\"-122.3643082\" /><node id=\"2428608447\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6591642\" lon=\"-122.3644132\" /><node id=\"2428608448\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6591661\" lon=\"-122.3643013\" /><node id=\"2428608449\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6591661\" lon=\"-122.3645069\" /><node id=\"2428608450\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6591664\" lon=\"-122.3643281\" /><node id=\"2428608451\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6591735\" lon=\"-122.364335\" /><node id=\"2428608452\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6591768\" lon=\"-122.3644134\" /><node id=\"2428608453\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6592003\" lon=\"-122.364719\" /><node id=\"2428608454\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6592047\" lon=\"-122.3647091\" /><node id=\"2428608455\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6592084\" lon=\"-122.3645085\" /><node id=\"2428608456\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6593284\" lon=\"-122.3642099\" /><node id=\"2428608457\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6593286\" lon=\"-122.3643884\" /><node id=\"2428608458\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6593581\" lon=\"-122.3643883\" /><node id=\"2428608459\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6593581\" lon=\"-122.3644021\" /><node id=\"2428608460\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6593916\" lon=\"-122.3643861\" /><node id=\"2428608461\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6593916\" lon=\"-122.364402\" /><node id=\"2428608462\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6594192\" lon=\"-122.3642096\" /><node id=\"2428608463\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6594192\" lon=\"-122.3642248\" /><node id=\"2428608464\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6594243\" lon=\"-122.3642248\" /><node id=\"2428608465\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6594245\" lon=\"-122.364386\" /><node id=\"2428608466\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6595757\" lon=\"-122.365697\" /><node id=\"2428608467\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6595761\" lon=\"-122.3660678\" /><node id=\"2428608468\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:50Z\" lat=\"47.6595895\" lon=\"-122.3643205\" /><node id=\"2428608469\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6595904\" lon=\"-122.3641977\" /><node id=\"2428608470\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6595964\" lon=\"-122.3648942\" /><node id=\"2428608471\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6595964\" lon=\"-122.3650188\" /><node id=\"2428608472\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6596357\" lon=\"-122.3643416\" /><node id=\"2428608473\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6596359\" lon=\"-122.3643212\" /><node id=\"2428608474\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.659654\" lon=\"-122.3643419\" /><node id=\"2428608475\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6596541\" lon=\"-122.3643278\" /><node id=\"2428608476\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6596606\" lon=\"-122.3641988\" /><node id=\"2428608477\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6596607\" lon=\"-122.3641847\" /><node id=\"2428608478\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6596788\" lon=\"-122.3641979\" /><node id=\"2428608479\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6596789\" lon=\"-122.364185\" /><node id=\"2428608480\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6596807\" lon=\"-122.3643282\" /><node id=\"2428608481\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6596817\" lon=\"-122.364198\" /><node id=\"2428608482\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6597208\" lon=\"-122.3651504\" /><node id=\"2428608483\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6597215\" lon=\"-122.3651152\" /><node id=\"2428608484\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.659738\" lon=\"-122.3656966\" /><node id=\"2428608485\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6597417\" lon=\"-122.3652324\" /><node id=\"2428608486\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6597432\" lon=\"-122.3651514\" /><node id=\"2428608487\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6597712\" lon=\"-122.3652607\" /><node id=\"2428608488\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6597717\" lon=\"-122.3652337\" /><node id=\"2428608489\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6597835\" lon=\"-122.3651179\" /><node id=\"2428608490\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6597837\" lon=\"-122.3651053\" /><node id=\"2428608491\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6598002\" lon=\"-122.3652619\" /><node id=\"2428608492\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6598006\" lon=\"-122.3652448\" /><node id=\"2428608493\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6598031\" lon=\"-122.365245\" /><node id=\"2428608494\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6598035\" lon=\"-122.3652255\" /><node id=\"2428608495\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.659867\" lon=\"-122.36582\" /><node id=\"2428608496\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6598767\" lon=\"-122.3660124\" /><node id=\"2428608497\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6598767\" lon=\"-122.3660671\" /><node id=\"2428608498\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6598794\" lon=\"-122.36582\" /><node id=\"2428608499\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6598796\" lon=\"-122.3660124\" /><node id=\"2428608500\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6598993\" lon=\"-122.3648943\" /><node id=\"2428608502\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6598993\" lon=\"-122.3650189\" /><node id=\"2428608504\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6599052\" lon=\"-122.3652299\" /><node id=\"2428608506\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6599075\" lon=\"-122.3651106\" /><node id=\"2428608508\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6600904\" lon=\"-122.3656072\" /><node id=\"2428608510\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6600928\" lon=\"-122.3654863\" /><node id=\"2428608511\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6601057\" lon=\"-122.3654869\" /><node id=\"2428608513\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6601058\" lon=\"-122.3654798\" /><node id=\"2428608515\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.660107\" lon=\"-122.3656233\" /><node id=\"2428608516\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:51Z\" lat=\"47.6601073\" lon=\"-122.3656079\" /><node id=\"2428608534\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6601683\" lon=\"-122.3656845\" /><node id=\"2428608550\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6601758\" lon=\"-122.3654992\" /><node id=\"2428608552\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6601761\" lon=\"-122.3654828\" /><node id=\"2428608555\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6601767\" lon=\"-122.3656661\" /><node id=\"2428608557\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6601774\" lon=\"-122.3656304\" /><node id=\"2428608565\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6601912\" lon=\"-122.3656963\" /><node id=\"2428608568\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6601925\" lon=\"-122.3656311\" /><node id=\"2428608577\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6602442\" lon=\"-122.365526\" /><node id=\"2428608580\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6602446\" lon=\"-122.3655022\" /><node id=\"2428608584\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6602553\" lon=\"-122.3655444\" /><node id=\"2428608587\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6602557\" lon=\"-122.3655265\" /><node id=\"2428608590\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6602678\" lon=\"-122.3657704\" /><node id=\"2428608592\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6602682\" lon=\"-122.3657491\" /><node id=\"2428608594\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6602756\" lon=\"-122.365804\" /><node id=\"2428608596\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6602767\" lon=\"-122.3657495\" /><node id=\"2428608603\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6603028\" lon=\"-122.3655464\" /><node id=\"2428608605\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6603108\" lon=\"-122.3656611\" /><node id=\"2428608607\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6603129\" lon=\"-122.3655569\" /><node id=\"2428608609\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6603268\" lon=\"-122.3656618\" /><node id=\"2428608611\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6603281\" lon=\"-122.3655958\" /><node id=\"2428608617\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6603437\" lon=\"-122.3658732\" /><node id=\"2428608619\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6603476\" lon=\"-122.3656744\" /><node id=\"2428608621\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6603594\" lon=\"-122.3656749\" /><node id=\"2428608623\" version=\"1\" changeset=\"17462348\" timestamp=\"2013-08-22T21:18:52Z\" lat=\"47.6603609\" lon=\"-122.3655972\" /><node id=\"67569701\" version=\"11\" changeset=\"20291912\" timestamp=\"2014-01-30T22:38:48Z\" lat=\"47.6588829\" lon=\"-122.3659509\" /><node id=\"67569713\" version=\"11\" changeset=\"20291912\" timestamp=\"2014-01-30T22:38:48Z\" lat=\"47.6589654\" lon=\"-122.3662172\" /><node id=\"1752970235\" version=\"2\" changeset=\"20291912\" timestamp=\"2014-01-30T22:38:48Z\" lat=\"47.6590182\" lon=\"-122.366265\" /><node id=\"1752970237\" version=\"2\" changeset=\"20291912\" timestamp=\"2014-01-30T22:38:48Z\" lat=\"47.659199\" lon=\"-122.3662674\" /><node id=\"1752970239\" version=\"2\" changeset=\"20291912\" timestamp=\"2014-01-30T22:38:48Z\" lat=\"47.659448\" lon=\"-122.3664895\" /><node id=\"2644688331\" version=\"1\" changeset=\"20291912\" timestamp=\"2014-01-30T22:38:48Z\" lat=\"47.6590595\" lon=\"-122.366272\" /><node id=\"2644688332\" version=\"1\" changeset=\"20291912\" timestamp=\"2014-01-30T22:38:48Z\" lat=\"47.658892\" lon=\"-122.3660303\" /><node id=\"53179274\" version=\"8\" changeset=\"20799217\" timestamp=\"2014-02-26T22:16:20Z\" lat=\"47.6589618\" lon=\"-122.3661047\" /><node id=\"60979342\" version=\"14\" changeset=\"20799217\" timestamp=\"2014-02-26T22:16:20Z\" lat=\"47.6592644\" lon=\"-122.3663091\"><tag k=\"bicycle\" v=\"yes\" /><tag k=\"highway\" v=\"crossing\" /></node><node id=\"67569707\" version=\"13\" changeset=\"20799217\" timestamp=\"2014-02-26T22:16:20Z\" lat=\"47.6589109\" lon=\"-122.3661095\"><tag k=\"bicycle\" v=\"yes\" /><tag k=\"highway\" v=\"crossing\" /></node><node id=\"2691856852\" version=\"1\" changeset=\"20799217\" timestamp=\"2014-02-26T22:16:20Z\" lat=\"47.6599489\" lon=\"-122.3661693\" /><node id=\"2889669694\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6593434\" lon=\"-122.3677214\" /><node id=\"2889669695\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6593434\" lon=\"-122.3678626\" /><node id=\"2889669696\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6594201\" lon=\"-122.3679358\" /><node id=\"2889669697\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6593489\" lon=\"-122.3680964\" /><node id=\"2889669698\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6593229\" lon=\"-122.368196\" /><node id=\"2889669699\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6592272\" lon=\"-122.3681026\" /><node id=\"2889669700\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6593489\" lon=\"-122.3680659\" /><node id=\"2889672601\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6593384\" lon=\"-122.3663818\" /><node id=\"2889672602\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6593544\" lon=\"-122.3663442\" /><node id=\"2889672604\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6595406\" lon=\"-122.3663035\" /><node id=\"2889672605\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6601868\" lon=\"-122.3663198\" /><node id=\"2889672606\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6604264\" lon=\"-122.366332\" /><node id=\"2889672607\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6605318\" lon=\"-122.3664316\" /><node id=\"2889672608\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.660572\" lon=\"-122.3663084\" /><node id=\"2889672609\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.659433\" lon=\"-122.3679719\" /><node id=\"2889672611\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.659529\" lon=\"-122.368597\" /><node id=\"2889672612\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6596282\" lon=\"-122.3685799\" /><node id=\"2889672613\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6596544\" lon=\"-122.3685796\" /><node id=\"2889672614\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6596942\" lon=\"-122.3686067\" /><node id=\"2889672615\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6597574\" lon=\"-122.3686711\" /><node id=\"2889672616\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6598071\" lon=\"-122.3687342\" /><node id=\"2889672617\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6598089\" lon=\"-122.3687704\" /><node id=\"2889672627\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6598766\" lon=\"-122.3687516\" /><node id=\"2889672628\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6598522\" lon=\"-122.3686349\" /><node id=\"2889672629\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:34Z\" lat=\"47.6598967\" lon=\"-122.3684902\" /><node id=\"2889672638\" version=\"1\" changeset=\"22643779\" timestamp=\"2014-05-30T18:39:35Z\" lat=\"47.6594353\" lon=\"-122.3682726\" /><node id=\"53179275\" version=\"9\" changeset=\"22649894\" timestamp=\"2014-05-31T05:00:19Z\" lat=\"47.6592681\" lon=\"-122.3661796\"><tag k=\"railway\" v=\"level_crossing\" /></node><node id=\"53230823\" version=\"8\" changeset=\"22649894\" timestamp=\"2014-05-31T05:00:19Z\" lat=\"47.6581579\" lon=\"-122.3650685\"><tag k=\"railway\" v=\"level_crossing\" /></node><node id=\"968442280\" version=\"4\" changeset=\"22649894\" timestamp=\"2014-05-31T05:00:19Z\" lat=\"47.6577707\" lon=\"-122.3647005\"><tag k=\"railway\" v=\"level_crossing\" /></node><node id=\"1752970227\" version=\"2\" changeset=\"22649894\" timestamp=\"2014-05-31T05:00:19Z\" lat=\"47.6564562\" lon=\"-122.363443\"><tag k=\"railway\" v=\"crossing\" /></node><node id=\"1752970233\" version=\"3\" changeset=\"22649894\" timestamp=\"2014-05-31T05:00:19Z\" lat=\"47.6589508\" lon=\"-122.3658502\"><tag k=\"railway\" v=\"level_crossing\" /></node><node id=\"2889672603\" version=\"2\" changeset=\"22649894\" timestamp=\"2014-05-31T05:00:19Z\" lat=\"47.6594333\" lon=\"-122.3663067\"><tag k=\"railway\" v=\"crossing\" /></node><node id=\"3096003519\" version=\"2\" changeset=\"26404331\" timestamp=\"2014-10-29T03:41:17Z\" lat=\"47.6588768\" lon=\"-122.3658409\" /><node id=\"3096003520\" version=\"2\" changeset=\"26404331\" timestamp=\"2014-10-29T03:41:17Z\" lat=\"47.6580699\" lon=\"-122.3650627\"><tag k=\"bicycle\" v=\"yes\" /><tag k=\"highway\" v=\"crossing\" /></node><node id=\"3096003521\" version=\"2\" changeset=\"26404331\" timestamp=\"2014-10-29T03:41:18Z\" lat=\"47.6569185\" lon=\"-122.3639625\"><tag k=\"bicycle\" v=\"yes\" /><tag k=\"highway\" v=\"crossing\" /></node><node id=\"3096003526\" version=\"2\" changeset=\"26404331\" timestamp=\"2014-10-29T03:41:18Z\" lat=\"47.6564734\" lon=\"-122.3633885\" /><node id=\"3155658707\" version=\"1\" changeset=\"26404331\" timestamp=\"2014-10-29T03:41:17Z\" lat=\"47.6577727\" lon=\"-122.3647576\"><tag k=\"bicycle\" v=\"yes\" /><tag k=\"highway\" v=\"crossing\" /></node><node id=\"3155658708\" version=\"1\" changeset=\"26404331\" timestamp=\"2014-10-29T03:41:17Z\" lat=\"47.6565579\" lon=\"-122.3635874\"><tag k=\"bicycle\" v=\"yes\" /><tag k=\"highway\" v=\"crossing\" /></node><node id=\"3155658709\" version=\"1\" changeset=\"26404331\" timestamp=\"2014-10-29T03:41:17Z\" lat=\"47.6564436\" lon=\"-122.3634831\" /><node id=\"3155658710\" version=\"1\" changeset=\"26404331\" timestamp=\"2014-10-29T03:41:17Z\" lat=\"47.6560549\" lon=\"-122.3629776\"><tag k=\"bicycle\" v=\"yes\" /><tag k=\"highway\" v=\"crossing\" /></node><node id=\"53112769\" version=\"6\" changeset=\"6214568\" timestamp=\"2010-10-29T06:02:01Z\" lat=\"47.6577839\" lon=\"-122.3618639\" /><node id=\"53112770\" version=\"6\" changeset=\"6214568\" timestamp=\"2010-10-29T06:01:54Z\" lat=\"47.6589481\" lon=\"-122.3618699\" /><node id=\"53161542\" version=\"7\" changeset=\"3341821\" timestamp=\"2009-12-10T14:03:06Z\" lat=\"47.6551865\" lon=\"-122.3610632\" /><node id=\"53161553\" version=\"7\" changeset=\"3341821\" timestamp=\"2009-12-10T14:03:06Z\" lat=\"47.6548576\" lon=\"-122.360748\" /><node id=\"504911046\" version=\"2\" changeset=\"3342188\" timestamp=\"2009-12-10T14:54:29Z\" lat=\"47.6549378\" lon=\"-122.360826\" /><node id=\"566896207\" version=\"2\" changeset=\"11244666\" timestamp=\"2012-04-09T21:02:41Z\" lat=\"47.6589519\" lon=\"-122.3601702\" /><node id=\"1709148214\" version=\"2\" changeset=\"18147994\" timestamp=\"2013-10-02T17:32:21Z\" lat=\"47.6589512\" lon=\"-122.3595835\"><tag k=\"traffic_calming\" v=\"island\" /></node><node id=\"53175453\" version=\"6\" changeset=\"18299984\" timestamp=\"2013-10-11T15:52:14Z\" lat=\"47.657813\" lon=\"-122.3595989\" /><node id=\"53189539\" version=\"7\" changeset=\"18299984\" timestamp=\"2013-10-11T15:52:14Z\" lat=\"47.657785\" lon=\"-122.3608169\" /><node id=\"53194835\" version=\"8\" changeset=\"18299984\" timestamp=\"2013-10-11T15:52:14Z\" lat=\"47.6589517\" lon=\"-122.3608148\" /><node id=\"53197490\" version=\"6\" changeset=\"18299984\" timestamp=\"2013-10-11T15:45:52Z\" lat=\"47.6560568\" lon=\"-122.3619115\" /><node id=\"566896206\" version=\"2\" changeset=\"18299984\" timestamp=\"2013-10-11T15:52:15Z\" lat=\"47.6578178\" lon=\"-122.3601724\" /><node id=\"2492631291\" version=\"1\" changeset=\"18299984\" timestamp=\"2013-10-11T15:52:14Z\" lat=\"47.6578119\" lon=\"-122.3607292\" /><node id=\"1833204869\" version=\"1\" changeset=\"12411058\" timestamp=\"2012-07-21T14:42:49Z\" lat=\"47.6627651\" lon=\"-122.3684124\" /><node id=\"29937454\" version=\"13\" changeset=\"11599644\" timestamp=\"2012-05-14T20:21:12Z\" lat=\"47.6613138\" lon=\"-122.3686151\"><tag k=\"bicycle\" v=\"yes\" /><tag k=\"highway\" v=\"crossing\" /></node><node id=\"29937456\" version=\"12\" changeset=\"11599644\" timestamp=\"2012-05-14T20:21:12Z\" lat=\"47.6612265\" lon=\"-122.3683765\" /><node id=\"29937457\" version=\"12\" changeset=\"11599644\" timestamp=\"2012-05-14T20:21:12Z\" lat=\"47.661124\" lon=\"-122.3681671\" /><node id=\"53148454\" version=\"6\" changeset=\"3420593\" timestamp=\"2009-12-21T15:01:30Z\" lat=\"47.6614588\" lon=\"-122.3671883\" /><node id=\"53191895\" version=\"5\" changeset=\"12411058\" timestamp=\"2012-07-21T14:42:50Z\" lat=\"47.6629259\" lon=\"-122.3685694\" /><node id=\"504911064\" version=\"1\" changeset=\"2565517\" timestamp=\"2009-09-22T07:08:20Z\" lat=\"47.6611653\" lon=\"-122.3685716\" /><node id=\"504911065\" version=\"1\" changeset=\"2565517\" timestamp=\"2009-09-22T07:08:20Z\" lat=\"47.6612195\" lon=\"-122.3686461\" /><node id=\"504911066\" version=\"1\" changeset=\"2565517\" timestamp=\"2009-09-22T07:08:20Z\" lat=\"47.6612356\" lon=\"-122.3686759\" /><node id=\"504911074\" version=\"1\" changeset=\"2565517\" timestamp=\"2009-09-22T07:08:20Z\" lat=\"47.6610931\" lon=\"-122.3684316\" /><node id=\"504916928\" version=\"1\" changeset=\"2565969\" timestamp=\"2009-09-22T07:39:05Z\" lat=\"47.6610923\" lon=\"-122.3686103\" /><node id=\"53105926\" version=\"6\" changeset=\"18299984\" timestamp=\"2013-10-11T15:56:32Z\" lat=\"47.6621955\" lon=\"-122.3678725\" /></osm>";
    private static final String BALLARD_RELATIONS_XML = "<?xml version='1.0' encoding='UTF-8' ?><osm version=\"0.6\" generator=\"OpenMapKit 0.1\" user=\"theoutpost\"><relation id=\"4065555\" version=\"6\" changeset=\"27281941\" timestamp=\"2014-12-06T02:57:17Z\"><member type=\"way\" ref=\"305036893\" role=\"outer\" /><member type=\"way\" ref=\"305036881\" role=\"outer\" /><member type=\"way\" ref=\"315997372\" role=\"outer\" /><member type=\"way\" ref=\"305077701\" role=\"outer\" /><member type=\"way\" ref=\"305067823\" role=\"outer\" /><member type=\"way\" ref=\"305039855\" role=\"outer\" /><tag k=\"landuse\" v=\"railway\" /></relation><relation id=\"4065613\" version=\"5\" changeset=\"27281941\" timestamp=\"2014-12-06T02:57:18Z\"><member type=\"way\" ref=\"305036893\" role=\"outer\" /><member type=\"way\" ref=\"305036881\" role=\"outer\" /><member type=\"way\" ref=\"315997372\" role=\"outer\" /><member type=\"way\" ref=\"305077701\" role=\"outer\" /><member type=\"way\" ref=\"305067823\" role=\"outer\" /><member type=\"way\" ref=\"305039855\" role=\"outer\" /><tag k=\"landuse\" v=\"railway\" /><tag k=\"type\" v=\"multipolygon\" /></relation><relation id=\"2183654\" version=\"27\" changeset=\"27490744\" timestamp=\"2014-12-15T20:06:32Z\"><member type=\"node\" ref=\"1152085626\" role=\"\" /><member type=\"way\" ref=\"38453959\" role=\"\" /><member type=\"way\" ref=\"38453957\" role=\"\" /><member type=\"way\" ref=\"163631481\" role=\"\" /><member type=\"way\" ref=\"163631457\" role=\"\" /><member type=\"way\" ref=\"163631459\" role=\"\" /><member type=\"way\" ref=\"163631462\" role=\"\" /><member type=\"way\" ref=\"4704945\" role=\"\" /><member type=\"way\" ref=\"55334404\" role=\"\" /><member type=\"way\" ref=\"55334386\" role=\"\" /><member type=\"way\" ref=\"55334389\" role=\"\" /><member type=\"way\" ref=\"13724604\" role=\"\" /><member type=\"way\" ref=\"317419786\" role=\"\" /><member type=\"way\" ref=\"317419800\" role=\"\" /><member type=\"way\" ref=\"314941568\" role=\"\" /><member type=\"way\" ref=\"314941569\" role=\"\" /><member type=\"way\" ref=\"4704943\" role=\"\" /><member type=\"way\" ref=\"34529462\" role=\"\" /><member type=\"way\" ref=\"8591489\" role=\"\" /><member type=\"way\" ref=\"163631478\" role=\"\" /><member type=\"way\" ref=\"242413136\" role=\"forward\" /><member type=\"way\" ref=\"242413135\" role=\"\" /><member type=\"way\" ref=\"242413138\" role=\"forward\" /><member type=\"way\" ref=\"34527908\" role=\"\" /><member type=\"way\" ref=\"4704942\" role=\"\" /><member type=\"way\" ref=\"283038445\" role=\"\" /><member type=\"way\" ref=\"49888204\" role=\"\" /><member type=\"way\" ref=\"49888212\" role=\"\" /><member type=\"way\" ref=\"283038442\" role=\"\" /><member type=\"way\" ref=\"283038449\" role=\"\" /><member type=\"way\" ref=\"313298244\" role=\"\" /><member type=\"way\" ref=\"315997374\" role=\"\" /><member type=\"way\" ref=\"313298245\" role=\"\" /><member type=\"way\" ref=\"163631463\" role=\"\" /><member type=\"way\" ref=\"34528146\" role=\"\" /><member type=\"way\" ref=\"283505160\" role=\"\" /><member type=\"way\" ref=\"283040790\" role=\"\" /><member type=\"way\" ref=\"283040789\" role=\"\" /><member type=\"way\" ref=\"283028558\" role=\"\" /><member type=\"way\" ref=\"34528139\" role=\"\" /><member type=\"way\" ref=\"163631482\" role=\"\" /><member type=\"way\" ref=\"163631468\" role=\"\" /><member type=\"way\" ref=\"163631466\" role=\"\" /><member type=\"way\" ref=\"315997367\" role=\"\" /><member type=\"way\" ref=\"114214913\" role=\"\" /><member type=\"way\" ref=\"315997368\" role=\"\" /><member type=\"way\" ref=\"163631458\" role=\"\" /><member type=\"way\" ref=\"163631454\" role=\"\" /><member type=\"way\" ref=\"315997370\" role=\"\" /><member type=\"way\" ref=\"4843301\" role=\"\" /><member type=\"way\" ref=\"305197030\" role=\"\" /><member type=\"way\" ref=\"163631464\" role=\"\" /><member type=\"way\" ref=\"163631473\" role=\"forward\" /><member type=\"way\" ref=\"163631472\" role=\"forward\" /><member type=\"way\" ref=\"4704877\" role=\"\" /><member type=\"way\" ref=\"39658955\" role=\"\" /><member type=\"way\" ref=\"163631488\" role=\"\" /><member type=\"way\" ref=\"6426102\" role=\"\" /><member type=\"way\" ref=\"163631487\" role=\"\" /><member type=\"way\" ref=\"142843343\" role=\"\" /><member type=\"way\" ref=\"263549159\" role=\"\" /><member type=\"way\" ref=\"6517019\" role=\"\" /><member type=\"way\" ref=\"13508387\" role=\"\" /><member type=\"way\" ref=\"13508385\" role=\"\" /><member type=\"way\" ref=\"13510030\" role=\"\" /><member type=\"way\" ref=\"315997699\" role=\"\" /><member type=\"way\" ref=\"315997716\" role=\"\" /><member type=\"way\" ref=\"259090353\" role=\"\" /><member type=\"way\" ref=\"242413137\" role=\"forward\" /><member type=\"way\" ref=\"242413139\" role=\"forward\" /><member type=\"way\" ref=\"37119926\" role=\"\" /><member type=\"way\" ref=\"7978647\" role=\"\" /><member type=\"way\" ref=\"4921433\" role=\"\" /><member type=\"way\" ref=\"283039317\" role=\"\" /><member type=\"way\" ref=\"283505164\" role=\"\" /><member type=\"way\" ref=\"7978669\" role=\"\" /><member type=\"way\" ref=\"313298247\" role=\"\" /><member type=\"way\" ref=\"241687841\" role=\"\" /><member type=\"way\" ref=\"313298246\" role=\"\" /><tag k=\"name\" v=\"Burke-Gilman Trail\" /><tag k=\"network\" v=\"lcn\" /><tag k=\"ref\" v=\"Burke-Gilman\" /><tag k=\"route\" v=\"bicycle\" /><tag k=\"type\" v=\"route\" /></relation><relation id=\"282850\" version=\"6\" changeset=\"14323471\" timestamp=\"2012-12-18T20:43:23Z\"><member type=\"way\" ref=\"159935907\" role=\"inner\" /><member type=\"way\" ref=\"4911374\" role=\"outer\" /><tag k=\"name\" v=\"Lake Union Ship Canal\" /><tag k=\"type\" v=\"multipolygon\" /><tag k=\"waterway\" v=\"riverbank\" /></relation><relation id=\"3132961\" version=\"1\" changeset=\"17304993\" timestamp=\"2013-08-11T15:38:12Z\"><member type=\"way\" ref=\"233133915\" role=\"outer\" /><member type=\"way\" ref=\"233133916\" role=\"inner\" /><tag k=\"building\" v=\"yes\" /><tag k=\"type\" v=\"multipolygon\" /></relation></osm>";
    private static final String FOUR_BS_MOD_TAGS_XML = "<?xml version='1.0' encoding='UTF-8' ?><osm version=\"0.6\" generator=\"OpenMapKit 0.1\" user=\"theoutpost\"><way action=\"modify\" id=\"234714287\" action=\"modify\" version=\"3\" changeset=\"28009821\" timestamp=\"2015-01-09T00:16:30Z\"><nd ref=\"2428608453\" /><nd ref=\"2428608437\" /><nd ref=\"2428608438\" /><nd ref=\"2428608448\" /><nd ref=\"2428608450\" /><nd ref=\"2428608451\" /><nd ref=\"2428608452\" /><nd ref=\"2428608447\" /><nd ref=\"2428608449\" /><nd ref=\"2428608455\" /><nd ref=\"2428608454\" /><nd ref=\"2428608453\" /><tag k=\"amenity\" v=\"bar\" /><tag k=\"building\" v=\"yes\" /><tag k=\"name\" v=\"Four B's\" /><tag k=\"testKey\" v=\"testValue\" /></way></osm>";
    private static final String SPDV_MOD_TAGS_XML = "<?xml version='1.0' encoding='UTF-8' ?><osm version=\"0.6\" generator=\"OpenMapKit 0.1\" user=\"theoutpost\"><node action=\"modify\" id=\"3203542408\" action=\"modify\" version=\"1\" changeset=\"27006416\" timestamp=\"2014-11-24T20:12:49Z\" lat=\"47.6690451\" lon=\"-122.3849681\"><tag k=\"addr:city\" v=\"Seattle\" /><tag k=\"addr:postcode\" v=\"98107\" /><tag k=\"addr:state\" v=\"WA\" /><tag k=\"addr:street\" v=\"NW Market St\" /><tag k=\"office\" v=\"company\" /><tag k=\"testKey\" v=\"testValue\" /><tag k=\"amenity\" v=\"this is some amenity\" /></node></osm>";
    
    public void setUp() throws Exception {
        super.setUp();
    }

    public void tearDown() throws Exception {
        super.tearDown();
    }
    
    public void testElementsToStringAsNull() throws Exception {
        String xml = OSMXmlWriter.elementsToString(null, USER);
        assertEquals(EMPTY_XML, xml);
    }
    
    public void testReadAndWriteBallardWays() throws Exception {
        InputStream in = getInstrumentation().getTargetContext().getResources().getAssets().open("test/osm/ballard_relation.osm");
        OSMDataSet ds = OSMXmlParser.parseFromInputStream(in);
        Collection<OSMWay> ways = ds.getWays().values();
        LinkedList<OSMElement> waysList = new LinkedList<>();
        for (OSMWay w : ways) {
            waysList.add(w);
        }
        String waysXml = OSMXmlWriter.elementsToString(waysList, USER);
        assertEquals(BALLARD_WAYS_XML, waysXml);
    }

    public void testReadAndWriteBallardNodes() throws Exception {
        InputStream in = getInstrumentation().getTargetContext().getResources().getAssets().open("test/osm/ballard_relation.osm");
        OSMDataSet ds = OSMXmlParser.parseFromInputStream(in);
        Collection<OSMNode> nodes = ds.getNodes().values();
        LinkedList<OSMElement> nodesList = new LinkedList<>();
        for (OSMNode n : nodes) {
            nodesList.add(n);
        }
        String nodesXml = OSMXmlWriter.elementsToString(nodesList, USER);
        assertEquals(BALLARD_NODES_XML, nodesXml);
    }

    public void testReadAndWriteBallardRelations() throws Exception {
        InputStream in = getInstrumentation().getTargetContext().getResources().getAssets().open("test/osm/ballard_relation.osm");
        OSMDataSet ds = OSMXmlParser.parseFromInputStream(in);
        Collection<OSMRelation> relations = ds.getRelations().values();
        LinkedList<OSMElement> relationsList = new LinkedList<>();
        for (OSMRelation r : relations) {
            relationsList.add(r);
        }
        String relationsXml = OSMXmlWriter.elementsToString(relationsList, USER);
        assertEquals(BALLARD_RELATIONS_XML, relationsXml);
    }
    
    public void testModifyingTagsFourBs() throws Exception {
        InputStream in = getInstrumentation().getTargetContext().getResources().getAssets().open("test/osm/ballard_relation.osm");
        OSMDataSet ds = OSMXmlParser.parseFromInputStream(in);

        Map<Long, OSMWay> wayMap = ds.getWays();
        OSMWay fourBs = wayMap.get((long)234714287);
        fourBs.addOrEditTag("testKey", "testValue");
        fourBs.addOrEditTag("amenity", "bar");
        fourBs.deleteTag("source");
        
        String xml = OSMXmlWriter.elementToString(fourBs, USER);
        assertEquals(FOUR_BS_MOD_TAGS_XML, xml);
    }
    
    public void testModifyingTagsSpatialDev() throws Exception {
        InputStream in = getInstrumentation().getTargetContext().getResources().getAssets().open("test/osm/spatialdev_small.osm");
        OSMDataSet ds = OSMXmlParser.parseFromInputStream(in);

        Map<Long, OSMNode> nodeMap = ds.getNodes();
        OSMNode spdv = nodeMap.get(Long.valueOf("3203542408"));
        spdv.addOrEditTag("testKey", "testValue");
        spdv.addOrEditTag("amenity", "this is some amenity");
        spdv.deleteTag("addr:housenumber");
        spdv.addOrEditTag("name", "SpatialDev");
        spdv.deleteTag("name");

        String xml = OSMXmlWriter.elementToString(spdv, USER);
        assertEquals(SPDV_MOD_TAGS_XML, xml);
    }
}
